@@ -1,6 +1,6 @@
 /*
-spnavcfg - an interactive GUI configurator for the spacenavd daemon.
-Copyright (C) 2007-2013 John Tsiombikas <nuclear@member.fsf.org>
+spacenavd - a free software replacement driver for 6dof space-mice.
+Copyright (C) 2007-2019 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,6 +25,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define MAX_BUTTONS		64
 #define MAX_CUSTOM		64
 
+enum {
+	LED_OFF		= 0,
+	LED_ON		= 1,
+	LED_AUTO	= 2
+};
+
 struct cfg {
 	float sensitivity, sens_trans[3], sens_rot[3];
 	int dead_threshold[MAX_AXES];
