@@ -1,6 +1,6 @@
 /*
 spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2012 John Tsiombikas <nuclear@member.fsf.org>
+Copyright (C) 2007-2019 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -35,9 +35,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "proto_x11.h"
 #include "client.h"
 #include "spnavd.h"
+#include "dev.h"
 #include "xdetect.h"
 #include "kbemu.h"
 
+#ifdef HAVE_XINPUT2_H
+#include <X11/Xatom.h>
+#include <X11/extensions/XInput2.h>
+#endif
+
 
 enum cmd_msg {
 	CMD_NONE,
@@ -88,11 +94,11 @@ int init_x11(void)
 		char *home, *buf;
 		if(!p || !p->pw_dir) {
 			if(!p) {
-				fprintf(stderr, "getpwuid failed: %s\n", strerror(errno));
+				logmsg(LOG_ERR, "getpwuid failed: %s\n", strerror(errno));
 			}
-			fprintf(stderr, "falling back to getting the home directory from the HOME env var...\n");
+			logmsg(LOG_WARNING, "falling back to getting the home directory from the HOME env var...\n");
 			if(!(home = getenv("HOME"))) {
-				fprintf(stderr, "HOME env var not found, using /tmp as a home directory...\n");
+				logmsg(LOG_WARNING, "HOME env var not found, using /tmp as a home directory...\n");
 				home = "/tmp";
 			}
 		} else {
@@ -105,12 +111,12 @@ int init_x11(void)
 	}
 
 	if(verbose) {
-		printf("trying to open X11 display \"%s\"\n", getenv("DISPLAY"));
-		printf("   XAUTHORITY=%s\n", getenv("XAUTHORITY"));
+		logmsg(LOG_INFO, "trying to open X11 display \"%s\"\n", getenv("DISPLAY"));
+		logmsg(LOG_INFO, "   XAUTHORITY=%s\n", getenv("XAUTHORITY"));
 	}
 
 	if(!(dpy = XOpenDisplay(0))) {
-		fprintf(stderr, "failed to open X11 display \"%s\"\n", getenv("DISPLAY"));
+		logmsg(LOG_ERR, "failed to open X11 display \"%s\"\n", getenv("DISPLAY"));
 
 		xdet_start();
 		return -1;
@@ -168,6 +174,8 @@ int init_x11(void)
 	kbemu_set_display(dpy);
 
 	xdet_stop();	/* stop X server detection if it was running */
+
+	drop_xinput();
 	return 0;
 }
 
@@ -178,7 +186,7 @@ void close_x11(void)
 
 	if(dpy && setjmp(jbuf) == 0) {
 		if(verbose) {
-			printf("closing X11 connection to display \"%s\"\n", getenv("DISPLAY"));
+			logmsg(LOG_INFO, "closing X11 connection to display \"%s\"\n", getenv("DISPLAY"));
 		}
 
 		/* first delete all the CommandEvent properties from all root windows */
@@ -346,6 +354,42 @@ void remove_client_window(Window win)
 	}
 }
 
+void drop_xinput(void)
+{
+#ifdef HAVE_XINPUT2_H
+	XIDeviceInfo *xidevs;
+	int i, num_devs;
+	static Atom atom_enabled;
+
+	if(!dpy) return;
+
+	if(!atom_enabled) {
+		atom_enabled = XInternAtom(dpy, "Device Enabled", False);
+	}
+
+	if(!(xidevs = XIQueryDevice(dpy, XIAllDevices, &num_devs))) {
+		logmsg(LOG_ERR, "failed to query XInput2 devices\n");
+		return;
+	}
+
+	for(i=0; i<num_devs; i++) {
+		struct device *dev = get_devices();
+		while(dev) {
+			if(strcmp(dev->name, xidevs[i].name) == 0 && xidevs[i].enabled) {
+				unsigned char zero = 0;
+				logmsg(LOG_INFO, "Removing device \"%s\" from X server\n", dev->name);
+
+				XIChangeProperty(dpy, xidevs[i].deviceid, atom_enabled, XA_INTEGER, 8, PropModeReplace, &zero, 1);
+				break;
+			}
+			dev = dev->next;
+		}
+	}
+	XIFreeDeviceInfo(xidevs);
+
+#endif	/* HAVE_XINPUT2_H */
+}
+
 
 /* X11 error handler */
 static int xerr(Display *dpy, XErrorEvent *err)
@@ -353,7 +397,7 @@ static int xerr(Display *dpy, XErrorEvent *err)
 	char buf[512];
 
 	if(verbose) {
-		fprintf(stderr, "xerr(%p, %p)\n", (void*)dpy, (void*)err);
+		logmsg(LOG_ERR, "xerr(%p, %p)\n", (void*)dpy, (void*)err);
 	}
 
 	if(err->error_code == BadWindow) {
@@ -363,7 +407,7 @@ static int xerr(Display *dpy, XErrorEvent *err)
 		remove_client_window((Window)err->resourceid);
 	} else {
 		XGetErrorText(dpy, err->error_code, buf, sizeof buf);
-		fprintf(stderr, "Caught unexpected X error: %s\n", buf);
+		logmsg(LOG_ERR, "Caught unexpected X error: %s\n", buf);
 	}
 	return 0;
 }
@@ -373,7 +417,7 @@ static int xerr(Display *dpy, XErrorEvent *err)
  */
 static int xioerr(Display *display)
 {
-	fprintf(stderr, "Lost the X server!\n");
+	logmsg(LOG_ERR, "Lost the X server!\n");
 	dpy = 0;
 	close_x11();
 	xdet_start();
